# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
pl custom locale file.
"""

translations = {
    "units": {"few_second": "kilka sekund"},
    # Relative time
    "ago": "{} temu",
    "from_now": "za {}",
    "after": "{0} po",
    "before": "{0} przed",
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "DD.MM.YYYY",
        "LL": "D MMMM YYYY",
        "LLL": "D MMMM YYYY HH:mm",
        "LLLL": "dddd, D MMMM YYYY HH:mm",
    },
}
